@@ -18,6 +18,7 @@
 #include <config.h>
 #endif
 
+#include <gconf/gconf-client.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <gconf/gconf.h>
@@ -26,6 +27,8 @@
 #include "drivemount.h"
 #include "properties.h"
 
+#define NEVER_SENSITIVE "never_sensitive"
+
 typedef struct _ResponseWidgets
 {
 	DriveData *dd;
@@ -45,6 +48,46 @@ static void set_widget_sensitivity_true_cb(GtkWidget *widget, GtkWidget *target)
 static gchar *remove_level_from_path(const gchar *path);
 static void sync_mount_base(DriveData *dd);
 
+/* set sensitive and setup NEVER_SENSITIVE appropriately */
+static void
+hard_set_sensitive (GtkWidget *w, gboolean sensitivity)
+{
+	gtk_widget_set_sensitive (w, sensitivity);
+	g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+			   GINT_TO_POINTER ( ! sensitivity));
+}
+
+
+/* set sensitive, but always insensitive if NEVER_SENSITIVE is set */
+static void
+soft_set_sensitive (GtkWidget *w, gboolean sensitivity)
+{
+	if (g_object_get_data (G_OBJECT (w), NEVER_SENSITIVE))
+		gtk_widget_set_sensitive (w, FALSE);
+	else
+		gtk_widget_set_sensitive (w, sensitivity);
+}
+
+
+static gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
+
 void
 properties_load(DriveData *dd)
 {
@@ -269,7 +312,7 @@ properties_show (BonoboUIComponent *uic,
 	GtkWidget *dialog;
 	GtkWidget *box;
 	GtkWidget *frame;
-	GtkWidget *hbox;
+	GtkWidget *hbox, *hbox2;
 	GtkWidget *vbox, *vbox1;
 	GtkWidget *fbox;
 	GtkWidget *label;
@@ -289,6 +332,7 @@ properties_show (BonoboUIComponent *uic,
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (dd->applet));
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
@@ -297,7 +341,7 @@ properties_show (BonoboUIComponent *uic,
 	size = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	
 	vbox = gtk_vbox_new(FALSE, 18);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_box_pack_start (GTK_BOX (box), vbox, FALSE, FALSE, 0);
 	gtk_widget_show(vbox);
 	
@@ -326,6 +370,11 @@ properties_show (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (cb_mount_activate), dd);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "mount_point")) {
+		hard_set_sensitive (label, FALSE);
+		hard_set_sensitive (widgets->mount_entry, FALSE);
+	}
+
 	hbox = gtk_hbox_new(FALSE, 12);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
@@ -336,19 +385,32 @@ properties_show (BonoboUIComponent *uic,
 	gtk_widget_show(label);
 	gtk_size_group_add_widget (size, label);
 
+	hbox2 = gtk_hbox_new(FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);
+	gtk_widget_show(hbox2);
+	
 	widgets->update_spin = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(dd->interval, 1.0, 300.0, 5, 1, 1)), 1, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), widgets->update_spin);
-	gtk_box_pack_start(GTK_BOX(hbox), widgets->update_spin, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox2), widgets->update_spin, FALSE, FALSE, 0);
 	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(widgets->update_spin),GTK_UPDATE_ALWAYS);
 	gtk_widget_show(widgets->update_spin);
 	g_signal_connect (G_OBJECT (widgets->update_spin), "focus_out_event",
 				  G_CALLBACK (spin_changed), dd);
+
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "interval")) {
+		hard_set_sensitive (label, FALSE);
+		hard_set_sensitive (widgets->update_spin, FALSE);
+	}
 	
 	label = gtk_label_new_with_mnemonic (_("seconds"));
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "interval")) {
+		hard_set_sensitive (label, FALSE);
+	}
+
 	hbox = gtk_hbox_new(FALSE, 12);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
@@ -365,8 +427,14 @@ properties_show (BonoboUIComponent *uic,
 	gtk_widget_show(widgets->omenu);
 	menu = gtk_menu_new();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(widgets->omenu), menu);
-	g_signal_connect (G_OBJECT (widgets->omenu), "changed",
-			  G_CALLBACK (omenu_changed), dd);
+
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "pixmap")) {
+		hard_set_sensitive (label, FALSE);
+		hard_set_sensitive (widgets->omenu, FALSE);
+	} else {
+		g_signal_connect (G_OBJECT (widgets->omenu), "changed",
+				  G_CALLBACK (omenu_changed), dd);
+	}
 	
 	/* This must be created before the menu items, so we can pass it to a callback */
 	fbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
@@ -423,6 +491,11 @@ properties_show (BonoboUIComponent *uic,
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "custom_icon_mounted")) {
+		hard_set_sensitive (label, FALSE);
+		hard_set_sensitive (widgets->icon_entry_in, FALSE);
+	}
+
 	hbox = gtk_hbox_new(FALSE, 12);
 	gtk_box_pack_start(GTK_BOX(fbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
@@ -446,10 +519,15 @@ properties_show (BonoboUIComponent *uic,
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "custom_icon_unmounted")) {
+		hard_set_sensitive (label, FALSE);
+		hard_set_sensitive (widgets->icon_entry_out, FALSE);
+	}
+
 	if(dd->device_pixmap < 0)
-		gtk_widget_set_sensitive(fbox, TRUE);
+		soft_set_sensitive(fbox, TRUE);
 	else
-		gtk_widget_set_sensitive(fbox, FALSE);
+		soft_set_sensitive(fbox, FALSE);
 #if 0
 	widgets->scale_toggle = gtk_check_button_new_with_mnemonic (_("_Scale size to panel"));
 	gtk_box_pack_start(GTK_BOX(vbox1), widgets->scale_toggle, FALSE, FALSE, 0);
@@ -465,6 +543,9 @@ properties_show (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT (widgets->eject_toggle), "toggled",
 			  G_CALLBACK (eject_toggled), dd);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "auto_eject"))
+		hard_set_sensitive (widgets->eject_toggle, FALSE);
+
 	widgets->automount_toggle = gtk_check_button_new_with_mnemonic (_("Use _automount friendly status test"));
 	gtk_box_pack_start(GTK_BOX(vbox1), widgets->automount_toggle, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->automount_toggle), dd->autofs_friendly);
@@ -473,6 +554,9 @@ properties_show (BonoboUIComponent *uic,
 			  G_CALLBACK (automount_toggled), dd);
 	gtk_widget_show_all(vbox);
 
+	if ( ! key_writable (PANEL_APPLET (dd->applet), "autofs_friendly"))
+		hard_set_sensitive (widgets->automount_toggle, FALSE);
+
 	widgets->dd = dd;
 	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(handle_response_cb), widgets);
 	gtk_widget_show_all(dialog);
@@ -499,14 +583,17 @@ help_cb (GtkDialog *dialog)
 
 	if (error) {
 		GtkWidget *error_dialog;
+		char *msg;
 
-		error_dialog = gtk_message_dialog_new (
-				NULL,
-				GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_CLOSE,
-				_("There was an error displaying help: %s"),
-				error->message);
+		msg = g_strdup_printf (_("There was an error displaying help: %s"),
+				       error->message);
+		error_dialog = hig_dialog_new (NULL /* parent */,
+					       GTK_DIALOG_DESTROY_WITH_PARENT,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("Error displaying help"),
+					       msg);
+		g_free (msg);
 
 		g_signal_connect (error_dialog, "response",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -535,13 +622,13 @@ handle_response_cb(GtkDialog *dialog, gint response, ResponseWidgets *widgets)
 static void
 set_widget_sensitivity_false_cb(GtkWidget *widget, GtkWidget *target)
 {
-	gtk_widget_set_sensitive(target, FALSE);
+	soft_set_sensitive(target, FALSE);
 }
 
 static void
 set_widget_sensitivity_true_cb(GtkWidget *widget, GtkWidget *target)
 {
-	gtk_widget_set_sensitive(target, TRUE);
+	soft_set_sensitive(target, TRUE);
 }
 
 static gchar *
