@@ -40,6 +40,8 @@
 
 #define STOCK_QUOTE(sq) ((StockQuote *)(sq))
 
+#define NEVER_SENSITIVE		"never_sensitive"
+
 	enum {
 		WHITE,
 		RED,
@@ -49,8 +51,10 @@
 	typedef struct {
 		char *price;
 		int pricelen;
+		int priceheight;
 		char *change;
 		int changelen;
+		int changeheight;
 		int color;
 	} StockQuote;
 	
@@ -67,6 +71,7 @@
 		gchar *fgcolor;
 		gchar *font;
 		gboolean buttons;
+		gboolean use_default_theme;
 		gint width;
 
 	} gtik_properties;
@@ -161,31 +166,79 @@
 	static AtkObject* gtik_vbox_accessible_ref_child(AtkObject *obj, gint i);
 	/* end accessibility funcs and vars */
 
+	/*-----------------------------------------------------------------*/
+	/* set sensitive and setup NEVER_SENSITIVE appropriately */
+	static void hard_set_sensitive (GtkWidget *w, gboolean sensitivity)
+	{
+		gtk_widget_set_sensitive (w, sensitivity);
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER ( ! sensitivity));
+	}
+
+
+	/*-----------------------------------------------------------------*/
+	/* set sensitive, but always insensitive if NEVER_SENSITIVE is set */
+	static void soft_set_sensitive (GtkWidget *w, gboolean sensitivity)
+	{
+		if (g_object_get_data (G_OBJECT (w), NEVER_SENSITIVE))
+			gtk_widget_set_sensitive (w, FALSE);
+		else
+			gtk_widget_set_sensitive (w, sensitivity);
+	}
+
+
+	/*-----------------------------------------------------------------*/
+	static gboolean key_writable (PanelApplet *applet, const char *key)
+	{
+		gboolean writable;
+		char *fullkey;
+		static GConfClient *client = NULL;
+		if (client == NULL)
+			client = gconf_client_get_default ();
+
+		fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+		writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+		g_free (fullkey);
+
+		return writable;
+	}
+
 	/*-----------------------------------------------------------------*/
 	static void load_fonts(StockData *stockdata)
 	{
 		GArray *quotes = stockdata->quotes;
 		PangoRectangle rect;
+		PangoContext *context;
 		int i;
 		
 		if (stockdata->my_font)
 			pango_font_description_free (stockdata->my_font);
-		stockdata->my_font = pango_font_description_from_string (stockdata->props.font);	
-		
-		if (!stockdata->my_font) {
-			stockdata->my_font = pango_font_description_from_string ("fixed 12");
+		stockdata->my_font = NULL;
+		if (stockdata->props.font ) {
+			stockdata->my_font = pango_font_description_from_string (stockdata->props.font);
+		}
+	
+		if (stockdata->props.use_default_theme || !stockdata->my_font) {
+			context = gtk_widget_get_pango_context (stockdata->applet);
+			stockdata->my_font = pango_font_description_copy (
+								pango_context_get_font_description (context));
 		}
 		pango_layout_set_font_description (stockdata->layout, stockdata->my_font);
+
 		/* make sure the cached strings widths are updated */
 		for (i=0; i< stockdata->setCounter; i++) {
 			pango_layout_set_text (stockdata->layout, 
 					       STOCK_QUOTE(quotes->data)[i].price, -1);
 			pango_layout_get_pixel_extents (stockdata->layout, NULL, &rect);
 			STOCK_QUOTE(quotes->data)[i].pricelen = rect.width;
+			STOCK_QUOTE(quotes->data)[i].priceheight = rect.height;
 			pango_layout_set_text (stockdata->layout, 
 					       STOCK_QUOTE(quotes->data)[i].change, -1);
 			pango_layout_get_pixel_extents (stockdata->layout, NULL, &rect);
 			STOCK_QUOTE(quotes->data)[i].changelen = rect.width;
+			STOCK_QUOTE(quotes->data)[i].changeheight = rect.height;
 		}
 	}
 
@@ -199,7 +252,7 @@ static void xfer_callback (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo
 		if (!configured(stockdata)) {
 			reSetOutputArray(stockdata);
 			setOutputArray(stockdata,
-				       _("No data available or properties not set"));
+				       _("Could not retrieve the stock data."));
 		}
 	}
 }
@@ -213,8 +266,10 @@ static gint updateOutput(gpointer data)
 	char *source_text_uri, *dest_text_uri;
 	GnomeVFSAsyncHandle *vfshandle;
 
-	if (stockdata->vfshandle != NULL)
-		return FALSE;
+	if (stockdata->vfshandle != NULL) {
+		gnome_vfs_async_cancel (stockdata->vfshandle);
+		stockdata->vfshandle = NULL;
+	}
 		
 	source_text_uri = g_strconcat("http://finance.yahoo.com/q?s=",
 				      stockdata->props.tik_syms,
@@ -229,6 +284,9 @@ static gint updateOutput(gpointer data)
 	dest_uri = gnome_vfs_uri_new(dest_text_uri);
 	dests = g_list_append(NULL, dest_uri);
 	g_free (dest_text_uri);
+	
+	reSetOutputArray(stockdata);
+	setOutputArray(stockdata,  _("Updating..."));
 
 	if (GNOME_VFS_OK !=
 	    gnome_vfs_async_xfer(&stockdata->vfshandle, sources, dests,
@@ -306,11 +364,13 @@ static gint updateOutput(gpointer data)
 									 NULL);
 		if (!stockdata->props.ucolor)
 			stockdata->props.ucolor = g_strdup ("#00ff00");
+			
+		stockdata->props.use_default_theme = panel_applet_gconf_get_bool(applet,
+									  "use_default_theme",
+									  NULL);
 		stockdata->props.font = panel_applet_gconf_get_string (applet,
 								       "font",
 								       NULL);
-		if (!stockdata->props.font)
-			stockdata->props.font = g_strdup ("fixed 12");
 				
 		stockdata->props.buttons = panel_applet_gconf_get_bool(applet,
 									  "buttons",
@@ -560,7 +620,7 @@ static gint updateOutput(gpointer data)
 		PangoRectangle logical_rect;
 		int	comp;
 		gint n = 0, totalLen2 = 0;
-		int start, end, width;
+		int start, end, width, max_height=0, y;
 
 		/* FOR COLOR */
 		int totalLoc;
@@ -569,8 +629,12 @@ static gint updateOutput(gpointer data)
 
 		totalLoc = 0;
 		totalLen = 0;
-	
-		gdk_gc_set_foreground( gc, &stockdata->gdkBGcolor );
+
+		if (stockdata->props.use_default_theme)
+			gdk_gc_set_foreground( gc, &stockdata->applet->style->base[GTK_WIDGET_STATE (stockdata->applet)] );
+		else
+			gdk_gc_set_foreground( gc, &stockdata->gdkBGcolor );
+		
 		gdk_draw_rectangle (stockdata->pixmap,
 				    gc, TRUE, 0,0,
 				    drawing_area->allocation.width,
@@ -580,17 +644,19 @@ static gint updateOutput(gpointer data)
 		
 		for (i=0; i< stockdata->setCounter; i++) {
 			totalLen += STOCK_QUOTE(quotes->data)[i].pricelen + 10;
-			
+			max_height = MAX (max_height, STOCK_QUOTE(quotes->data)[i].priceheight);
 			if (stockdata->props.output == FALSE) {
 				if (*(STOCK_QUOTE(quotes->data)[i].change)) {
 					totalLen += STOCK_QUOTE(quotes->data)[i].changelen 
 						+ 10;
+					max_height = MAX (max_height, STOCK_QUOTE(quotes->data)[i].changeheight);
 					
 				}
 			}
 			
 		}
 		
+		y = MAX((drawing_area->allocation.height - max_height)/2, 0);
 		comp = 0 - totalLen;
 
 		if (stockdata->MOVE == 1) { stockdata->MOVE = 0; } 
@@ -598,7 +664,6 @@ static gint updateOutput(gpointer data)
 
 		if (stockdata->MOVE == 1) {
 
-
 		  if (stockdata->props.scroll == TRUE) {
 			if (stockdata->location > comp) {
 				stockdata->location -= stockdata->delta;	
@@ -624,16 +689,21 @@ static gint updateOutput(gpointer data)
 		}
 
 		for (i=0;i<stockdata->setCounter;i++) {
-
+			
+			
 			/* COLOR */
-			if (STOCK_QUOTE(quotes->data)[i].color == GREEN) {
-				gdk_gc_set_foreground( gc, &stockdata->gdkUcolor );
-			}
-			else if (STOCK_QUOTE(quotes->data)[i].color == RED) {
-				gdk_gc_set_foreground( gc, &stockdata->gdkDcolor );
-			}
+			if (stockdata->props.use_default_theme)
+				gdk_gc_set_foreground( gc, &stockdata->applet->style->fg[GTK_WIDGET_STATE (stockdata->applet)] );
 			else {
-				gdk_gc_set_foreground( gc, &stockdata->gdkFGcolor );
+				if (STOCK_QUOTE(quotes->data)[i].color == GREEN) {
+					gdk_gc_set_foreground( gc, &stockdata->gdkUcolor );
+				}
+				else if (STOCK_QUOTE(quotes->data)[i].color == RED) {
+					gdk_gc_set_foreground( gc, &stockdata->gdkDcolor );
+				}
+				else {
+					gdk_gc_set_foreground( gc, &stockdata->gdkFGcolor );
+				}
 			}
 
 			start = stockdata->location + totalLoc;
@@ -645,7 +715,7 @@ static gint updateOutput(gpointer data)
 					       STOCK_QUOTE(quotes->data)[i].price,
 					       -1);	
 				gdk_draw_layout (stockdata->pixmap, gc,
-					 start , 3,
+					 start , y,
 					 layout);
 			}
 			totalLoc += STOCK_QUOTE(quotes->data)[i].pricelen + 10;
@@ -659,7 +729,7 @@ static gint updateOutput(gpointer data)
 						STOCK_QUOTE(quotes->data)[i].change, -1);
 					gdk_draw_layout (stockdata->pixmap,
 					     		gc, stockdata->location + totalLoc,
-					     		3, layout);
+					     		y, layout);
 				}
 				totalLoc += STOCK_QUOTE(quotes->data)[i].changelen + 10;
 			}
@@ -799,7 +869,7 @@ static gint updateOutput(gpointer data)
 		GtkWidget *button;
 
 		button = GTK_WIDGET(data);
-		gtk_widget_set_sensitive(button,TRUE);
+		soft_set_sensitive(button,TRUE);
 	}
 
 	/*-----------------------------------------------------------------*/
@@ -843,26 +913,26 @@ static gint updateOutput(gpointer data)
 		
 	}
 	
-	static gboolean width_changed (GtkWidget *spin, GdkEventFocus *event, StockData *stockdata)
+	static void width_changed (GtkSpinButton *spin, StockData *stockdata)
  	{
 		PanelApplet *applet = PANEL_APPLET (stockdata->applet);
-		gint width, height;
+		gint width;
 		
 		width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 		if (width < 1)
-			return FALSE;
+			return;
 			
 		if (stockdata->props.width == width)
-			return FALSE;
+			return;
 			
 		stockdata->props.width = width;
 		panel_applet_gconf_set_int (applet, "width", 
 					    stockdata->props.width, NULL);
-		height = panel_applet_get_size (applet) - 4;			    
+				    
 		gtk_drawing_area_size(GTK_DRAWING_AREA (stockdata->drawing_area),
-						stockdata->props.width,height);
+						stockdata->props.width,-1);
 	
-		return FALSE;
+		return;
 	}
 	
 	static void
@@ -911,6 +981,31 @@ static gint updateOutput(gpointer data)
 					     
 	}	
 	
+	static void
+	def_font_toggled (GtkToggleButton *button, gpointer data)
+	{
+		StockData *stockdata = data;
+		PanelApplet *applet = PANEL_APPLET (stockdata->applet);
+		GtkWidget *fonts_widget, *colors_widget;
+		gboolean toggle;
+		
+		toggle = gtk_toggle_button_get_active (button);
+		if (toggle != stockdata->props.use_default_theme)
+			stockdata->props.use_default_theme = toggle;
+		else
+			return;
+		panel_applet_gconf_set_bool (applet,"use_default_theme",
+					     stockdata->props.use_default_theme, NULL);
+					     
+		fonts_widget = g_object_get_data (G_OBJECT (button), "fonts_hbox");
+		colors_widget = g_object_get_data (G_OBJECT (button), "colors_hbox");
+		soft_set_sensitive (GTK_WIDGET (fonts_widget), 
+					             !stockdata->props.use_default_theme);
+		soft_set_sensitive (GTK_WIDGET (colors_widget), 
+					             !stockdata->props.use_default_theme);
+		load_fonts(stockdata);
+	}
+	
 	static void
 	add_symbol (GtkEntry *entry, gpointer data)
 	{
@@ -953,6 +1048,7 @@ static gint updateOutput(gpointer data)
 					    
 		gtk_entry_set_text (entry, "");
 		g_free (symbol);
+		updateOutput(stockdata);
 		
 	}
 	
@@ -1036,8 +1132,31 @@ static gint updateOutput(gpointer data)
 		else
 			panel_applet_gconf_set_string (applet, "tik_syms",
 					               "", NULL);
+		updateOutput(stockdata);
 		
 	}
+	
+	static void
+	selection_changed (GtkTreeSelection *selection, gpointer data) 
+	{
+		soft_set_sensitive (GTK_WIDGET (data), 
+						    gtk_tree_selection_get_selected (selection, NULL, NULL));
+						    
+	}
+	
+	static void
+	entry_changed (GtkEditable *editable, gpointer data)
+	{
+		gboolean str = TRUE;
+		gchar *string;
+		
+		string = gtk_editable_get_chars (editable, 0, -1);
+		if (!string || !g_utf8_strlen (string, -1))
+			str = FALSE;
+		soft_set_sensitive (GTK_WIDGET (data), str);
+		if (string) g_free (string);
+	}
+		
 
 	void font_cb(GnomeFontPicker *gfp, const gchar *font_name, gpointer data) {
 		StockData *stockdata = data;
@@ -1120,14 +1239,15 @@ static gint updateOutput(gpointer data)
 		GtkWidget *label;
 		GtkWidget *entry;
 		static GtkWidget *button;
+		GtkSizeGroup *size_group;
 
 		model = gtk_list_store_new (1, G_TYPE_STRING);
 		list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
-		gtk_widget_set_size_request (list, 70, -1);
+		/*gtk_widget_set_size_request (list, 70, -1);*/
 		g_object_unref (G_OBJECT (model));
 		
-		/*selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-  		gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);*/
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
+		/*gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);*/
   		
   		cell_renderer = gtk_cell_renderer_text_new ();
   		column = gtk_tree_view_column_new_with_attributes ("hello",
@@ -1155,34 +1275,47 @@ static gint updateOutput(gpointer data)
 		
 		hbox = gtk_hbox_new (FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (mainhbox), hbox, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("Current _Stocks"));
+		label = gtk_label_new_with_mnemonic (_("Current _stocks:"));
 		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), list);
 		
 		gtk_box_pack_start(GTK_BOX(mainhbox),swindow,TRUE,TRUE,0);
 		
-		hbox = gtk_hbox_new (FALSE, 6);
+		hbox = gtk_hbox_new (FALSE, 12);
 		gtk_box_pack_start (GTK_BOX (mainhbox), hbox, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic(_("_New Symbol:"));
+		label = gtk_label_new_with_mnemonic (_("_New symbol:"));
 		gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);
 		entry = gtk_entry_new();
 		g_object_set_data(G_OBJECT(entry),"list",(gpointer)list);
 		gtk_box_pack_start(GTK_BOX(hbox),entry,TRUE,TRUE,0);
 		g_signal_connect (G_OBJECT (entry), "activate",
 				  G_CALLBACK (add_symbol), stockdata);		
-		set_relation(entry, GTK_LABEL(label));		
+		set_relation (entry, GTK_LABEL (label));
+		size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);		
 		button = gtk_button_new_with_mnemonic(_("_Add"));
+		gtk_size_group_add_widget (size_group, button);
 		g_object_set_data (G_OBJECT (button), "entry", entry);
 		gtk_box_pack_start(GTK_BOX(hbox),button,TRUE,TRUE,0);
+		soft_set_sensitive (button, FALSE);
 		g_signal_connect (G_OBJECT (button), "clicked",
 				  G_CALLBACK (add_button_clicked), stockdata);
+		g_signal_connect (G_OBJECT (entry), "changed",
+					  G_CALLBACK (entry_changed), button);
 		button = gtk_button_new_with_mnemonic(_("_Remove"));
+		gtk_size_group_add_widget (size_group, button);
 		g_object_set_data (G_OBJECT (button), "list", list);
 		g_signal_connect (G_OBJECT (button), "clicked",
 				  G_CALLBACK (remove_symbol), stockdata);
 		gtk_box_pack_start(GTK_BOX(hbox),button,FALSE,FALSE,0);
+		soft_set_sensitive (button, FALSE);
+		g_signal_connect (G_OBJECT (selection), "changed",
+					   G_CALLBACK (selection_changed), button);
 
 		gtk_widget_show_all(mainhbox);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "tik_syms"))
+			hard_set_sensitive (mainhbox, FALSE);
+
 		return(mainhbox);
 
 	}
@@ -1257,13 +1390,13 @@ static gint updateOutput(gpointer data)
 		StockData * stockdata = data;
 		GtkWidget * notebook;
 		GtkWidget * vbox, *behav_vbox, *appear_vbox;
-		GtkWidget * vbox2;
-		GtkWidget *hbox, *hbox2;
+		GtkWidget * vbox2, *color_vbox;
+		GtkWidget *hbox, *hbox2, *hbox3, *font_hbox;
 		GtkWidget * label, *spin, *check;
 		GtkWidget *color;
 		GtkWidget *font;
 		GtkWidget *option, *menu, *menuitem;
-		GtkSizeGroup *size;
+		GtkSizeGroup *size, *size2;
 
 		int ur,ug,ub, dr,dg,db; 
 				
@@ -1284,13 +1417,21 @@ static gint updateOutput(gpointer data)
 						  	     NULL);
 		gtk_window_set_screen (GTK_WINDOW (stockdata->pb),
 				       gtk_widget_get_screen (stockdata->applet));
+		gtk_dialog_set_default_response (GTK_DIALOG (stockdata->pb), 
+		                                 GTK_RESPONSE_CLOSE);
+		gtk_dialog_set_has_separator (GTK_DIALOG (stockdata->pb), FALSE);
+		gtk_container_set_border_width (GTK_CONTAINER (stockdata->pb), 5);
+		gtk_widget_show (GTK_DIALOG (stockdata->pb)->vbox);
 		notebook = gtk_notebook_new ();
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (stockdata->pb)->vbox), notebook,
 				    TRUE, TRUE, 0);
+		gtk_widget_show (notebook);
 
 		hbox = symbolManager(stockdata);
 		label = gtk_label_new (_("Symbols"));
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), hbox, label);
+		gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
+		gtk_widget_show_all (hbox);
 		
 		behav_vbox = gtk_vbox_new (FALSE, 18);
 		gtk_container_set_border_width (GTK_CONTAINER (behav_vbox), 12);
@@ -1299,31 +1440,39 @@ static gint updateOutput(gpointer data)
 		
 		vbox = create_hig_catagory (behav_vbox, _("Update"));
 		
-		hbox2 = gtk_hbox_new (FALSE, 6);
+		hbox2 = gtk_hbox_new (FALSE, 12);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("Stock update Fre_quency in minutes:"));
+		label = gtk_label_new_with_mnemonic (_("Stock update fre_quency:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
+		
+		hbox3 = gtk_hbox_new (FALSE, 6);
+		gtk_box_pack_start (GTK_BOX (hbox2), hbox3, FALSE, FALSE, 0);
 		spin = gtk_spin_button_new_with_range (1, 1000, 1);
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), stockdata->props.timeout);
 		g_signal_connect (G_OBJECT (spin), "focus_out_event",
 					   G_CALLBACK (timeout_cb), stockdata);
-		gtk_box_pack_start (GTK_BOX (hbox2), spin, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox3), spin, FALSE, FALSE, 0);
+		label = gtk_label_new (_("minutes"));
+		gtk_box_pack_start (GTK_BOX (hbox3), label, FALSE, FALSE, 0);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "timeout"))
+			hard_set_sensitive (hbox2, FALSE);
 		
 		vbox = create_hig_catagory (behav_vbox, _("Scrolling"));
 		
-		hbox2 = gtk_hbox_new (FALSE, 6);
+		hbox2 = gtk_hbox_new (FALSE, 12);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
 		label = gtk_label_new_with_mnemonic (_("_Scroll speed:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		menu = gtk_menu_new ();
-		menuitem = gtk_menu_item_new_with_mnemonic (_("_Slow"));
+		menuitem = gtk_menu_item_new_with_label (_("Slow"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		menuitem = gtk_menu_item_new_with_mnemonic (_("_Medium"));
+		menuitem = gtk_menu_item_new_with_label (_("Medium"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		menuitem = gtk_menu_item_new_with_mnemonic (_("_Fast"));
+		menuitem = gtk_menu_item_new_with_label (_("Fast"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);		
 		option = gtk_option_menu_new ();
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), option);
@@ -1337,6 +1486,9 @@ static gint updateOutput(gpointer data)
 			gtk_option_menu_set_history (GTK_OPTION_MENU (option), 1);
 		else
 			gtk_option_menu_set_history (GTK_OPTION_MENU (option), 0);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "scroll_speed"))
+			hard_set_sensitive (hbox2, FALSE);
 		
 		hbox2 = gtk_hbox_new (FALSE, 6);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
@@ -1346,6 +1498,9 @@ static gint updateOutput(gpointer data)
 				  	  G_CALLBACK (scroll_toggled), stockdata);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
 							   stockdata->props.buttons);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "buttons"))
+			hard_set_sensitive (hbox2, FALSE);
 							   
 		hbox2 = gtk_hbox_new (FALSE, 6);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
@@ -1355,60 +1510,108 @@ static gint updateOutput(gpointer data)
 				           G_CALLBACK (rtl_toggled), stockdata);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
 							   !stockdata->props.scroll);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "scroll"))
+			hard_set_sensitive (hbox2, FALSE);
+							   
+		gtk_widget_show_all (behav_vbox);
 							   
 		appear_vbox = gtk_vbox_new (FALSE, 18);
 		gtk_container_set_border_width (GTK_CONTAINER (appear_vbox), 12);
 		label = gtk_label_new (_("Appearance"));
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), appear_vbox, label);
+		gtk_widget_show_all(appear_vbox);
+		gtk_widget_show_all (notebook);
 		
 		size = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 		
 		vbox = create_hig_catagory (appear_vbox, _("Display"));
+		gtk_widget_show_all (vbox);
 		
 		hbox2 = gtk_hbox_new (FALSE, 6);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
+		check = gtk_check_button_new_with_mnemonic (_("Displa_y only symbols and price"));
+		gtk_box_pack_start (GTK_BOX (hbox2), check, FALSE, FALSE, 0);
+		g_signal_connect (G_OBJECT (check), "toggled",
+				           G_CALLBACK (output_toggled), stockdata);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+							   stockdata->props.output);
+		gtk_widget_show_all (hbox2);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "output"))
+			hard_set_sensitive (hbox2, FALSE);
+
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
 		label = gtk_label_new_with_mnemonic (_("_Width:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
+		gtk_widget_show_all (hbox2);
+		
+		hbox3 = gtk_hbox_new (FALSE, 6);
+		gtk_box_pack_start (GTK_BOX (hbox2), hbox3, FALSE, FALSE, 0);
 		spin = gtk_spin_button_new_with_range (20, 500, 10);
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), stockdata->props.width);
-		g_signal_connect (G_OBJECT (spin), "focus_out_event",
+		g_signal_connect (G_OBJECT (spin), "value_changed",
 					   G_CALLBACK (width_changed), stockdata);
-		gtk_box_pack_start (GTK_BOX (hbox2), spin, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox3), spin, FALSE, FALSE, 0);
+		label = gtk_label_new (_("pixels"));
+		gtk_box_pack_start (GTK_BOX (hbox3), label, FALSE, FALSE, 0);
+		gtk_widget_show_all (hbox2);	
 		
+		g_object_unref (size);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "width"))
+			hard_set_sensitive (hbox2, FALSE);
+		
+		vbox = create_hig_catagory (appear_vbox, _("Font and Colors"));
+		size = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+		size2= gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+				
 		hbox2 = gtk_hbox_new (FALSE, 6);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		check = gtk_check_button_new_with_mnemonic(_("Displa_y only symbols and price"));
+		check = gtk_check_button_new_with_mnemonic (_("Use _default theme font and colors"));
 		gtk_box_pack_start (GTK_BOX (hbox2), check, FALSE, FALSE, 0);
 		g_signal_connect (G_OBJECT (check), "toggled",
-				           G_CALLBACK (output_toggled), stockdata);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
-							   stockdata->props.output);
-							   
-		hbox2 = gtk_hbox_new (FALSE, 6);
+				           G_CALLBACK (def_font_toggled), stockdata);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+							   stockdata->props.use_default_theme);
+		gtk_widget_show_all (hbox2);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "use_default_theme"))
+			hard_set_sensitive (hbox2, FALSE);
+					   
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		font_hbox = hbox2;
+		g_object_set_data (G_OBJECT (check), "fonts_hbox", hbox2);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("_Font"));
+		label = gtk_label_new_with_mnemonic (_("_Font:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
 		font = gnome_font_picker_new ();
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), font);
-		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (font),
+		if (stockdata->props.font)
+			gnome_font_picker_set_font_name (GNOME_FONT_PICKER (font),
 						 stockdata->props.font);
 		gtk_box_pack_start (GTK_BOX (hbox2), font, FALSE, FALSE, 0);
 		g_signal_connect (G_OBJECT (font), "font_set",
                 		  G_CALLBACK (font_cb), stockdata);
-                		  
-                g_object_unref (size);
-		
-		vbox = create_hig_catagory (appear_vbox, _("Colors"));
-		size = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-					   
-		hbox2 = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("Stock _raised"));
+                gtk_size_group_add_widget (size2, font);
+		gtk_widget_show_all (hbox2);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "font"))
+			hard_set_sensitive (hbox2, FALSE);
+               	
+		color_vbox = gtk_vbox_new (FALSE, 6);
+		g_object_set_data (G_OBJECT (check), "colors_hbox", color_vbox);
+		gtk_box_pack_start (GTK_BOX (vbox), color_vbox, TRUE, TRUE, 0);
+				   
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (color_vbox), hbox2, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic (_("Stock _raised:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
@@ -1418,12 +1621,16 @@ static gint updateOutput(gpointer data)
 		gnome_color_picker_set_i8(GNOME_COLOR_PICKER (color), 
 					  ur, ug, ub, 255);
 		gtk_box_pack_start (GTK_BOX (hbox2), color, FALSE, FALSE, 0);
+		gtk_size_group_add_widget (size2, color);
 		g_signal_connect(G_OBJECT(color), "color_set",
 				G_CALLBACK(ucolor_set_cb), stockdata);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "ucolor"))
+			hard_set_sensitive (hbox2, FALSE);
 				
-		hbox2 = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("Stock _lowered"));
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (color_vbox), hbox2, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic (_("Stock _lowered:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
@@ -1433,12 +1640,16 @@ static gint updateOutput(gpointer data)
 		gnome_color_picker_set_i8(GNOME_COLOR_PICKER (color), 
 					  ur, ug, ub, 255);
 		gtk_box_pack_start (GTK_BOX (hbox2), color, FALSE, FALSE, 0);
+		gtk_size_group_add_widget (size2, color);
 		g_signal_connect(G_OBJECT(color), "color_set",
 				G_CALLBACK(dcolor_set_cb), stockdata);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "dcolor"))
+			hard_set_sensitive (hbox2, FALSE);
 				
-		hbox2 = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("Stock _unchanged"));
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (color_vbox), hbox2, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic (_("Stock _unchanged:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
@@ -1448,12 +1659,16 @@ static gint updateOutput(gpointer data)
 		gnome_color_picker_set_i8(GNOME_COLOR_PICKER (color), 
 					  ur, ug, ub, 255);
 		gtk_box_pack_start (GTK_BOX (hbox2), color, FALSE, FALSE, 0);
+		gtk_size_group_add_widget (size2, color);
 		g_signal_connect(G_OBJECT(color), "color_set",
 				G_CALLBACK(fgcolor_set_cb), stockdata);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "fgcolor"))
+			hard_set_sensitive (hbox2, FALSE);
 				
-		hbox2 = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
-		label = gtk_label_new_with_mnemonic (_("_Background"));
+		hbox2 = gtk_hbox_new (FALSE, 12);
+		gtk_box_pack_start (GTK_BOX (color_vbox), hbox2, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic (_("_Background:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
 		gtk_size_group_add_widget (size, label);
@@ -1463,13 +1678,23 @@ static gint updateOutput(gpointer data)
 		gnome_color_picker_set_i8(GNOME_COLOR_PICKER (color), 
 					  ur, ug, ub, 255);
 		gtk_box_pack_start (GTK_BOX (hbox2), color, FALSE, FALSE, 0);
+		gtk_size_group_add_widget (size2, color);
 		g_signal_connect(G_OBJECT(color), "color_set",
 				G_CALLBACK(bgcolor_set_cb), stockdata);
+
+		if ( ! key_writable (PANEL_APPLET (stockdata->applet), "bgcolor"))
+			hard_set_sensitive (hbox2, FALSE);
+				
+		gtk_widget_show_all (vbox);
 				
 		g_object_unref (G_OBJECT (size));
+		g_object_unref (G_OBJECT (size2));
 	
 		gtk_widget_show_all(stockdata->pb);
 		
+		soft_set_sensitive (font_hbox, !stockdata->props.use_default_theme);
+		soft_set_sensitive (color_vbox, !stockdata->props.use_default_theme);
+		
 		g_signal_connect (G_OBJECT (stockdata->pb), "response",
 				  G_CALLBACK (response_cb), stockdata);
 
@@ -1501,6 +1726,31 @@ static gint updateOutput(gpointer data)
 				      stockdata->props.width,size-4);
     	
     	}
+    	
+    	static void
+    	style_set_cb (GtkWidget *widget, GtkStyle *previous_style, gpointer data)
+    	{
+    		StockData *stockdata = data;
+    		
+    		if (stockdata->props.use_default_theme);
+    			load_fonts (stockdata);
+    	}
+
+	static gboolean 
+	key_press_cb (GtkWidget *widget, GdkEventKey *event, StockData *stockdata)
+	{
+		switch (event->keyval) {	
+		case GDK_u:
+			if (event->state == GDK_CONTROL_MASK)
+				updateOutput(stockdata);
+			return TRUE;
+		default:
+			break;
+		}
+
+		return FALSE;
+
+	}
     
 	static const BonoboUIVerb gtik_applet_menu_verbs [] = {
         	BONOBO_UI_UNSAFE_VERB ("Props", properties_cb),
@@ -1516,12 +1766,13 @@ static gint updateOutput(gpointer data)
 		StockData *stockdata;
 		GtkWidget * vbox;
 		GtkWidget * frame;
-		gint height;
+		GtkWidget *event;
 
 		gnome_vfs_init();
 		
 		gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-money.png");
 		panel_applet_add_preferences (applet, "/schemas/apps/gtik/prefs", NULL);
+		panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 		
 		access_stock = stockdata = g_new0 (StockData, 1);
 		stockdata->applet = GTK_WIDGET (applet);
@@ -1552,22 +1803,24 @@ static gint updateOutput(gpointer data)
 		stockdata->tooltips = gtk_tooltips_new ();
 		gtk_tooltips_set_tip(stockdata->tooltips, stockdata->leftButton, _("Skip forward"), NULL);
 		gtk_tooltips_set_tip(stockdata->tooltips, stockdata->rightButton, _("Skip backward"), NULL);
-		gtk_tooltips_set_tip(stockdata->tooltips, stockdata->applet, _("Stock Ticker\nGet continuously updated stock quotes"), NULL);
-
-
+		
 		frame = gtk_frame_new(NULL);
 		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
+		
+		event = gtk_event_box_new ();
+		gtk_widget_show (event);
+		gtk_container_add (GTK_CONTAINER(frame), event);
+		gtk_tooltips_set_tip(stockdata->tooltips, event, _("Stock Ticker\nGet continuously updated stock quotes"), NULL);
 
 		access_drawing_area = stockdata->drawing_area = GTK_WIDGET (custom_drawing_area_new());
-		height = panel_applet_get_size (applet) - 4;
 		gtk_drawing_area_size(GTK_DRAWING_AREA (stockdata->drawing_area),
-						stockdata->props.width,height);
+						stockdata->props.width,-1);
 
 		gtk_widget_show(stockdata->drawing_area);
 
 		stockdata->layout = gtk_widget_create_pango_layout (stockdata->drawing_area, "");
 
-		gtk_container_add(GTK_CONTAINER(frame),stockdata->drawing_area);
+		gtk_container_add(GTK_CONTAINER(event),stockdata->drawing_area);
 		
 		gtk_widget_show(frame);
 
@@ -1592,6 +1845,12 @@ static gint updateOutput(gpointer data)
 
 		g_signal_connect (G_OBJECT (applet), "change_size",
 				  G_CALLBACK (applet_change_size), stockdata);
+				  
+		g_signal_connect (G_OBJECT (applet), "style_set",
+					   G_CALLBACK (style_set_cb), stockdata);
+					   
+		g_signal_connect (applet, "key_press_event",
+					  G_CALLBACK (key_press_cb), stockdata);
 
 		gtk_widget_show (GTK_WIDGET (applet));
 		
@@ -1723,7 +1982,7 @@ static gint updateOutput(gpointer data)
 		var4 = strtok(NULL,"");
 
 		if (!var3 || !var4)
-			return data;
+			return NULL;
 
 		if (var3[0] == '+') { 
 #if 0
@@ -1765,10 +2024,20 @@ static gint updateOutput(gpointer data)
 		pango_layout_set_text (stockdata->layout, price, -1);
 		pango_layout_get_pixel_extents (stockdata->layout, NULL, &rect);
 		quote.pricelen = rect.width;
-		quote.change = g_strdup(change);
-		pango_layout_set_text (stockdata->layout, change, -1);
-		pango_layout_get_pixel_extents (stockdata->layout, NULL, &rect);
-		quote.changelen = rect.width;
+		quote.priceheight = rect.height;
+		if (change) {
+			quote.change = g_strdup(change);
+			pango_layout_set_text (stockdata->layout, change, -1);
+			pango_layout_get_pixel_extents (stockdata->layout, NULL, &rect);
+			quote.changelen = rect.width;
+			quote.changeheight = rect.height;
+		}
+		else {
+			quote.change = g_strdup("");
+			quote.changelen = 0;
+			quote.changeheight = 0;
+		}
+		
 
 #if 0
 		g_message("Param1: %s\nPrice: %s\nChange: %s\nColor: %d\n\n", param1, price, change, quote.color);
