@@ -18,11 +18,17 @@ static unsigned needed_cpu_flags =
 (1 << GLIBTOP_CPU_USER) +
 (1 << GLIBTOP_CPU_IDLE);
 
+static unsigned needed_page_flags = 
+(1 << GLIBTOP_SWAP_PAGEIN) +
+(1 << GLIBTOP_SWAP_PAGEOUT);
+
 static unsigned needed_mem_flags =
 (1 << GLIBTOP_MEM_USED) +
 (1 << GLIBTOP_MEM_FREE);
 
-static unsigned needed_swap_flags = 0;
+static unsigned needed_swap_flags =
+(1 << GLIBTOP_SWAP_USED) +
+(1 << GLIBTOP_SWAP_FREE);
 
 static unsigned needed_loadavg_flags =
 (1 << GLIBTOP_LOADAVG_LOADAVG);
@@ -75,9 +81,56 @@ GetLoad (int Maximum, int data [4], LoadGraph *g)
 }
 
 void
-GetMemory (int Maximum, int data [4], LoadGraph *g)
+GetPage (int Maximum, int data [3], LoadGraph *g)
 {
-    int user, shared, buffer, free;
+    static int max = 100; /* guess at maximum page rate (= in + out) */
+    static u_int64_t lastin = 0;
+    static u_int64_t lastout = 0;
+    int in, out, idle;
+
+    glibtop_swap swap;
+	
+    glibtop_get_swap (&swap);
+	
+    assert ((swap.flags & needed_page_flags) == needed_page_flags);
+
+    if ((lastin > 0) && (lastin < swap.pagein)) {
+	in = swap.pagein - lastin;
+    }
+    else {
+	in = 0;
+    }
+    lastin = swap.pagein;
+
+    if ((lastout > 0) && (lastout < swap.pageout)) {
+	out = swap.pageout - lastout;
+    }
+    else {
+	out = 0;
+    }
+    lastout = swap.pageout;
+
+    if ((in + out) > max) {
+	/* Maximum page rate has increased. Change the scale without
+	   any indication whatsoever to the user (not a nice thing to
+	   do). */
+	max = in + out;
+    }
+
+    in   = rint (Maximum * ((float)in / max));
+    out  = rint (Maximum * ((float)out / max));
+    idle = Maximum - in - out;
+
+    data [0] = in;
+    data [1] = out;
+    data [2] = idle;
+}
+
+void
+GetMemory (int Maximum, int data [5], LoadGraph *g)
+{
+    int user, shared, buffer, cached;
+    unsigned long tmp_user;
 
     glibtop_mem mem;
 	
@@ -85,17 +138,18 @@ GetMemory (int Maximum, int data [4], LoadGraph *g)
 	
     assert ((mem.flags & needed_mem_flags) == needed_mem_flags);
 
-    user = mem.used - mem.buffer - mem.shared;
-	
-    user    = rint (Maximum * (float)user   / mem.total);
+    tmp_user = mem.used - mem.buffer - mem.shared - mem.cached;
+   
+    user    = rint (Maximum * (float)tmp_user / mem.total);
     shared  = rint (Maximum * (float)mem.shared / mem.total);
     buffer  = rint (Maximum * (float)mem.buffer / mem.total);
-    free    = rint (Maximum * (float)mem.free / mem.total);
-
+    cached = rint (Maximum * (float)mem.cached / mem.total);
+    
     data [0] = user;
     data [1] = shared;
     data [2] = buffer;
-    data [3] = free;
+    data[3] = cached;
+    data [4] = Maximum-user-shared-buffer-cached;
 }
 
 void
@@ -136,6 +190,10 @@ GetLoadAvg (int Maximum, int data [2], LoadGraph *g)
 	
     assert ((loadavg.flags & needed_loadavg_flags) == needed_loadavg_flags);
 
+	index = 0;
+	
+/* i need to figure out what this means... */
+/*
     switch (g->prop_data->loadavg_type) {
     case LOADAVG_1:
 	index = 0;
@@ -150,10 +208,11 @@ GetLoadAvg (int Maximum, int data [2], LoadGraph *g)
 	g_assert_not_reached ();
 	return;
     }
-
+*/
+/*
     if (g->prop_data_ptr->adj_data [2])
 	max_loadavg = (float) g->prop_data_ptr->adj_data [2];
-
+*/
     if (loadavg.loadavg [index] > max_loadavg)
 	loadavg.loadavg [index] = max_loadavg;
 
@@ -165,19 +224,26 @@ GetLoadAvg (int Maximum, int data [2], LoadGraph *g)
 }
 
 void
-GetNet (int Maximum, int data [4], LoadGraph *g)
+GetNet (int Maximum, int data [5], LoadGraph *g)
 {
 #define SLIP_COUNT	0
 #define PPP_COUNT	1
 #define ETH_COUNT       2
 #define OTHER_COUNT	3
 #define COUNT_TYPES	4
-    int fields[4], present[COUNT_TYPES], delta[COUNT_TYPES], i;
-    static int ticks, past[COUNT_TYPES];
-    static char *netdevfmt;
+    gulong inbytes, outbytes;
+    gulong present[COUNT_TYPES];
+    int delta[COUNT_TYPES], i;
+    static int ticks = 0;
+    static gulong past[COUNT_TYPES] = {0};
+    static char *netdevfmt = NULL;
     char *cp, buffer[256];
-    int found = 0;
     FILE *fp;
+    
+    /* FIXME: this is hosed for multiple applets with the static variables */
+
+    /* FIXME: This function is generally incredibly evil -George
+       (too lazy to rewrite it though) */
 
     /*
      * We use the maximum number of bits we've seen come through to scale
@@ -194,8 +260,8 @@ GetNet (int Maximum, int data [4], LoadGraph *g)
     {
 	FILE *fp = popen("uname -r", "r");	/* still wins if /proc isn't */
 
-	/* pre-linux-2.2 format -- transmit packet count in 8th field */
-	netdevfmt = "%d %d %*d %*d %*d %d %*d %d %*d %*d %*d %*d %d";
+	/* pre-linux-2.2 format -- transmit packet count in 8th field (bytes in 1st and 7th) */
+	netdevfmt = "%lu %*d %*d %*d %*d %*d %lu";
 
 	if (!fp)
 	    return;
@@ -210,8 +276,8 @@ GetNet (int Maximum, int data [4], LoadGraph *g)
 	    }
 
 	    if (major >= 2 && minor >= 2)
-		/* Linux 2.2 -- transmit packet count in 10th field */
-		netdevfmt = "%d %d %*d %*d %*d %d %*d %*d %*d %*d %d %*d %d";
+		/* Linux 2.2 -- transmit packet count in 10th field (bytes in 1st and 9th) */
+		netdevfmt = "%lu %*d %*d %*d %*d %*d %*d %*d %lu";
 	    pclose(fp);
 	}
     }
@@ -220,55 +286,51 @@ GetNet (int Maximum, int data [4], LoadGraph *g)
     if (!fp)
 	return;
 
-    memset(present, '0', sizeof(present));
+    for (i = 0; i < COUNT_TYPES; i++)
+	    present[i] = 0;
 
     while (fgets(buffer, sizeof(buffer) - 1, fp))
     {
-	int	*resp;
+	int index;
 
 	for (cp = buffer; *cp == ' '; cp++)
 	    continue;
 
-	resp = present + OTHER_COUNT;
-	switch (cp[0]) {
-	case 'l':		
-		if (cp[1] == 'o')
-			continue;
-		break;
-	case 's':
-		if (!strncmp (cp+1, "lip", 3))
-			resp = present + SLIP_COUNT;
-		break;
-	case 'p':
-		if (!strncmp (cp+1, "pp", 2))
-			resp = present + PPP_COUNT;
-		break;
-	case 'e':
-		if (!strncmp (cp+1, "th", 2))
-			resp = present + ETH_COUNT;
-		break;
+	if (strncmp (cp, "lo", 2) == 0) {
+		continue;
+	} else if (strncmp (cp, "slip", 4) == 0) {
+		index = SLIP_COUNT;
+	} else if (strncmp (cp, "ppp", 3) == 0) {
+		index = PPP_COUNT;
+	} else if (strncmp (cp, "eth", 3) == 0) {
+		index = ETH_COUNT;
+	} else {
+      		index = OTHER_COUNT;
 	}
 
 	if ((cp = strchr(buffer, ':')))
 	{
 	    cp++;
-	    if (sscanf(cp, netdevfmt,
-		       fields, fields+1, fields+2, 
-		       fields+3,&found)>4)
-		*resp += fields[0] + fields[2];
+	    if ( sscanf(cp, netdevfmt, &inbytes, &outbytes) == 2 )
+		present[index] += inbytes + outbytes;
 	}
     }
 
     fclose(fp);
 
-    memset(data, '\0', sizeof(data));
+    for (i = 0; i < 5; i++)
+	    data[i] = 0;
     if (ticks++ > 0)		/* avoid initial spike */
     {
 	int total = 0;
 
 	for (i = 0; i < COUNT_TYPES; i++)
 	{
-	    delta[i] = (present[i] - past[i]);
+	    /* protect against weirdness */
+	    if (present[i] >= past[i])
+		    delta[i] = (present[i] - past[i]);
+	    else
+		    delta[i] = 0;
 	    total += delta[i];
 	}
 	if (total > max)
@@ -284,7 +346,10 @@ GetNet (int Maximum, int data [4], LoadGraph *g)
 	       data[0], data[1], data[2], Maximum);
 #endif
     }
-    memcpy(past, present, sizeof(present));
+    data[4] = Maximum - data[3] - data[2] - data[1] - data[0];
+
+    for (i = 0; i < COUNT_TYPES; i++)
+	    past[i] = present[i];
 }
 
 
